@@ -3,13 +3,13 @@
 #' \code{specan} measures acoustic parameters on acoustic signals for which the start and end times 
 #' are provided. 
 #' @usage specan(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast = TRUE, path = NULL, 
-#' pb = TRUE)
+#' pb = TRUE, ovlp = 50, ff.method = "seewave", wn = "hanning")
 #' @param X Data frame with the following columns: 1) "sound.files": name of the .wav 
 #' files, 2) "sel": number of the selections, 3) "start": start time of selections, 4) "end": 
 #' end time of selections. The ouptut of \code{\link{manualoc}} or \code{\link{autodetec}} can
 #' be used as the input data frame.
 #' @param bp A numeric vector of length 2 for the lower and upper limits of a 
-#'   frequency bandpass filter (in kHz) or "frange" to indicate that values in low.f 
+#'   frequency bandpass filter (in kHz) or "frange" to indicate that values in low.f
 #'   and high.f columns will be used as bandpass limits. Default is c(0, 22).
 #' @param wl A numeric vector of length 1 specifying the spectrogram window length. Default is 512.
 #' @param threshold amplitude threshold (\%) for fundamental frequency and 
@@ -21,32 +21,57 @@
 #' substantially increases performance (~9 times faster).
 #' @param path Character string containing the directory path where the sound files are located. 
 #' If \code{NULL} (default) then the current working directory is used.
-#' @param pb Logical argument to control progress bar. Default is \code{TRUE}. Note that progress bar is only used
+#' @param pb Logical argument to control progress bar and messages. Default is \code{TRUE}. Note that progress bar is only used
 #' when parallel = 1.
-#' @return Data frame with the following acoustic parameters: 
+#' @param ovlp Numeric vector of length 1 specifying \% of overlap between two 
+#'   consecutive windows, used for fundamental frequency (using \code{\link[seewave]{fund}} or \code{\link[tuneR]{FF}}) and dominant frequency (using \code{\link[seewave]{dfreq}}). 
+#'   Default is 50. 
+#' @param ff.method Character. Selects the method used to calculate the fundamental
+#' frequency. Either 'tuneR' (using \code{\link[tuneR]{FF}}) or 'seewave' (using 
+#' \code{\link[seewave]{fund}}). Default is 'seewave'. Use trackfreqs to decide which method works the best. 
+#' 'tuneR' performs faster (and seems to be more accurate) than 'seewave'.
+#' @param wn Character vector of length 1 specifying window name. Default is hanning'. 
+#' See function \code{\link[seewave]{ftwindow}} for more options.
+#' @return Data frame with 'sound.files' and 'selec' as in the input data frame, plus the following acoustic parameters: 
 #' \itemize{
-#'    \item \code{duration}: length of signal
-#'    \item \code{meanfreq}: mean frequency (in kHz)
-#'    \item \code{sd}: standard deviation of frequency 
-#'    \item \code{median}: median frequency (in kHz) 
-#'    \item \code{Q25}: first quantile (in kHz) 
-#'    \item \code{Q75}: third quantile (in kHz) 
-#'    \item \code{IQR}: interquantile range (in kHz) 
-#'    \item \code{skew}: skewness (see note in \code{\link[seewave]{specprop}} description) 
-#'    \item \code{kurt}:  kurtosis (see note in \code{\link[seewave]{specprop}} description)
-#'    \item \code{sp.ent}: spectral entropy 
-#'    \item \code{sfm}: spectral flatness 
-#'    \item \code{mode}: mode frequency
-#'    \item \code{centroid}: frequency centroid (see \code{\link[seewave]{specprop}})
-#'    \item \code{peakf}: peak frequency (frequency with highest energy) 
-#'    \item \code{meanfun}: average of fundamental frequency measured across acoustic signal 
-#'    \item \code{minfun}: minimum fundamental frequency measured across acoustic signal 
-#'    \item \code{maxfun}: maximum fundamental frequency measured across acoustic signal 
-#'    \item \code{meandom}: average of dominant frequency measured across acoustic signal 
-#'    \item \code{mindom}: minimum of dominant frequency measured across acoustic signal
-#'    \item \code{maxdom}: maximum of dominant frequency measured across acoustic signal 
-#'    \item \code{dfrange}: range of dominant frequency measured across acoustic signal 
-#'    \item \code{modindx}: modulation index. Calculated as the accumulated absolute 
+#'    \item \code{duration}: length of signal (in s)
+#'    \item \code{meanfreq}: mean frequency. Weighted average of frequency by amplitude (in kHz)
+#'    \item \code{sd}: standard deviation of frequency weighted by amplitude
+#'    \item \code{freq.median}: median frequency. The frequency at which the signal is divided in two frequency
+#'    intervals of equal energy (in kHz) 
+#'    \item \code{freq.Q25}: first quartile frequency. The frequency at which the signal is divided in two 
+#'    frequency intervals of 25\% and 75\% energy respectively (in kHz) 
+#'    \item \code{freq.Q75}: third quartile frequency. The frequency at which the signal is divided in two
+#'    frequency intervals of 75\% and 25\% energy respectively (in kHz) 
+#'    \item \code{freq.IQR}: interquartile frequency range. Frequency range between 'freq.Q25' and 'freq.Q75' 
+#'    (in kHz) 
+#'    \item \code{time.median}: median time. The time at which the signal is divided in two time
+#'    intervals of equal energy (in s) 
+#'    \item \code{time.Q25}: first quartile time. The time at which the signal is divided in two
+#'time intervals of 25\% and 75\% energy respectively (in s). See \code{\link[seewave]{acoustat}}
+#'    \item \code{time.Q75}: third quartile time. The time at which the signal is divided in two
+#'    time intervals of 75\% and 25\% energy respectively (in s). See \code{\link[seewave]{acoustat}}
+#'    \item \code{time.IQR}: interquartile time range. Time range between 'time.Q25' and 'time.Q75' 
+#'    (in s). See \code{\link[seewave]{acoustat}}
+#'    \item \code{skew}: skewness. Asymmetry of the spectrum (see note in \code{\link[seewave]{specprop}} description) 
+#'    \item \code{kurt}: kurtosis. Peakedness of the spectrum (see note in \code{\link[seewave]{specprop}} description)
+#'    \item \code{sp.ent}: spectral entropy. Energy distribution of the frequency spectrum. Pure tone ~ 0; 
+#'    noisy ~ 1. See \code{\link[seewave]{sh}}
+#'    \item \code{time.ent}: time entropy. Energy distribution on the time envelope. Pure tone ~ 0; 
+#'    noisy ~ 1. See \code{\link[seewave]{th}}
+#'    \item \code{entropy}: spectral entropy. Product of time and spectral entropy \code{sp.ent * time.ent}. 
+#'    See \code{\link[seewave]{H}}
+#'    \item \code{sfm}: spectral flatness. Similar to sp.ent (Pure tone ~ 0; 
+#'    noisy ~ 1). See \code{\link[seewave]{sfm}}
+#'    \item \code{peakf}: peak frequency. Frequency with highest energy
+#'    \item \code{meanfun}: average of fundamental frequency measured across the acoustic signal 
+#'    \item \code{minfun}: minimum fundamental frequency measured across the acoustic signal 
+#'    \item \code{maxfun}: maximum fundamental frequency measured across the acoustic signal 
+#'    \item \code{meandom}: average of dominant frequency measured across the acoustic signal 
+#'    \item \code{mindom}: minimum of dominant frequency measured across the acoustic signal
+#'    \item \code{maxdom}: maximum of dominant frequency measured across the acoustic signal 
+#'    \item \code{dfrange}: range of dominant frequency measured across the acoustic signal 
+#'    \item \code{modindx}: modulation index. Calculated as the cumulative absolute 
 #'      difference between adjacent measurements of dominant frequencies divided
 #'      by the dominant frequency range
 #'    \item \code{startdom}:  dominant frequency measurement at the start of the signal 
@@ -56,7 +81,7 @@
 #' @export
 #' @name specan
 #' @details The ouptut of \code{\link{manualoc}} or \code{\link{autodetec}} can be used 
-#'  directly without any additional modification. The function measures 25 acoustic parameters (if fast = \code{TRUE}) on 
+#'  directly without any additional modification. The function measures 29 acoustic parameters (if fast = \code{TRUE}) on 
 #'  each selection in the data frame. Most parameters are produced internally by 
 #'  \code{\link[seewave]{specprop}}, \code{\link[seewave]{fpeaks}}, \code{\link[seewave]{fund}},
 #'  and \code{\link[seewave]{dfreq}} from the package seewave. 
@@ -83,7 +108,7 @@
 #last modification on jul-5-2016 (MAS)
 
 specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast = TRUE, 
-                   path = NULL, pb = TRUE){
+                   path = NULL, pb = TRUE, ovlp = 50, ff.method = "seewave", wn = "hanning"){
   
   #check path to working directory
   if(!is.null(path))
@@ -108,12 +133,14 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
   #if any start higher than end stop
   if(any(X$end - X$start<0)) stop(paste("The start is higher than the end in", length(which(X$end - X$start<0)), "case(s)"))  
   
-  #if any selections longer than 20 secs stop
-  if(any(X$end - X$start>20)) stop(paste(length(which(X$end - X$start>20)), "selection(s) longer than 20 sec"))  
-  options( show.error.messages = TRUE)
+  #if any selections longer than 20 secs warning
+  if(any(X$end - X$start>20)) warning(paste(length(which(X$end - X$start>20)), "selection(s) longer than 20 sec"))
+  
+  #if ff.method argument  
+  if(!any(ff.method == "seewave", ff.method == "tuneR")) stop(paste("ff.method", ff.method, "is not recognized")) 
   
   # bp checking
-  if(bp != "frange")
+  if(bp[1] != "frange")
   {if(!is.vector(bp)) stop("'bp' must be a numeric vector of length 2") else{
     if(!length(bp) == 2) stop("'bp' must be a numeric vector of length 2")} 
   } else
@@ -145,44 +172,68 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
   spFUN <- function(i, X, bp, wl, threshold) { 
     r <- tuneR::readWave(as.character(X$sound.files[i]), from = X$start[i], to = X$end[i], units = "seconds") 
     
-    if(bp == "frange") bp <- c(X$low.f[i], X$high.f[i])
-    b<- bp 
-    
+    if(bp[1] == "frange") bp <- c(X$low.f[i], X$high.f[i])
+
     b<- bp #in case bp its higher than can be due to sampling rate
     if(b[2] > ceiling(r@samp.rate/2000) - 1) b[2] <- ceiling(r@samp.rate/2000) - 1 
     
     
     #frequency spectrum analysis
-    songspec <- seewave::spec(r, f = r@samp.rate, plot = FALSE)
+    songspec <- seewave::spec(r, f = r@samp.rate, plot = FALSE, wl = wl, wn = wn, flim = b)
     analysis <- seewave::specprop(songspec, f = r@samp.rate, flim = b, plot = FALSE)
+
+    #acoustat
+    m <- sspectro(r, f = r@samp.rate, wl = wl, ovlp = ovlp, wn = wn)
+    fl <- b * nrow(m) * 2000/r@samp.rate
+    m <- m[(fl[1]:fl[2]) + 1, ]
+    if(is.vector(m)) m <- t(as.matrix(m))
+    time <- seq(0, length(r)/r@samp.rate, length.out = ncol(m))
+    t.cont <- apply(m, MARGIN = 2, FUN = sum)
+    t.cont <- t.cont/sum(t.cont)
+    t.cont.cum <- cumsum(t.cont)
+    t.quartiles <- sapply(c(0.25, 0.5, 0.75), function(x) time[length(t.cont.cum[t.cont.cum <= 
+                                                                                       x]) + 1])
     
     #save parameters
     meanfreq <- analysis$mean/1000
     sd <- analysis$sd/1000
-    median <- analysis$median/1000
-    Q25 <- analysis$Q25/1000
-    Q75 <- analysis$Q75/1000
-    IQR <- analysis$IQR/1000
+    freq.median <- analysis$median/1000
+    freq.Q25 <- analysis$Q25/1000
+    freq.Q75 <- analysis$Q75/1000
+    freq.IQR <- analysis$IQR/1000
+    time.ent <- th(cbind(time, t.cont))
+    time.median <- t.quartiles[2]
+    time.Q25 <- t.quartiles[1]
+    time.Q75 <- t.quartiles[3]
+    time.IQR <- t.quartiles[3] - t.quartiles[1]
     skew <- analysis$skewness
     kurt <- analysis$kurtosis
     sp.ent <- analysis$sh
+    entropy <- sp.ent * time.ent
     sfm <- analysis$sfm
-    mode <- analysis$mode/1000
-    centroid <- analysis$cent/1000
     
     #Frequency with amplitude peaks 
     if(!fast) #only if fast is TRUE
-      peakf <- seewave::fpeaks(songspec, f = r@samp.rate, wl = 512, nmax = 3, plot = FALSE)[1, 1] else peakf <- NA
+      peakf <- seewave::fpeaks(songspec, f = r@samp.rate, wl = wl, nmax = 3, plot = FALSE)[1, 1] else peakf <- NA
     
     #Fundamental frequency parameters
-    ff <- seewave::fund(r, f = r@samp.rate, ovlp = 50, threshold = threshold, 
-                        fmax = b[2] * 1000, plot = FALSE)[, 2]
+    if(ff.method == "seewave")
+    ff <- seewave::fund(r, f = r@samp.rate, ovlp = ovlp, threshold = threshold, 
+                        fmax = b[2] * 1000, plot = FALSE)[, 2] else {
+                          if(!r@stereo)
+                            ff <- tuneR::FF(tuneR::periodogram(r, width = wl, 
+                                                               overlap = wl*ovlp/100), peakheight = (100 - threshold) / 100)/1000
+                          else
+                            ff <- tuneR::FF(tuneR::periodogram(mono(r, "left"), width = wl, 
+                                                               overlap = wl*ovlp/100), peakheight = (100 - threshold) / 100)/1000
+                        }
+    
     meanfun<-mean(ff, na.rm = TRUE)
     minfun<-min(ff, na.rm = TRUE)
     maxfun<-max(ff, na.rm = TRUE)
     
     #Dominant frecuency parameters
-    y <- seewave::dfreq(r, f = r@samp.rate, wl = wl, ovlp = 0, plot = FALSE, threshold = threshold, bandpass = b * 1000, fftw = TRUE)[, 2]
+    y <- seewave::dfreq(r, f = r@samp.rate, wl = wl, ovlp = ovlp, plot = FALSE, threshold = threshold, bandpass = b * 1000, fftw = TRUE)[, 2]
     meandom <- mean(y, na.rm = TRUE)
     if(length(y[y > 0 & !is.na(y)]) > 0) mindom <- min(y[y > 0 & !is.na(y)]) else mindom <- NA
     maxdom <- max(y, na.rm = TRUE)
@@ -202,12 +253,14 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
     if(mindom==maxdom) modindx<-0 else modindx <- mean(changes, na.rm = TRUE)/dfrange
     
     #save results
-    if(fast) return(data.frame(sound.files = X$sound.files[i], selec = X$selec[i], duration, meanfreq, sd, median, Q25, Q75, IQR, skew, kurt, sp.ent, sfm, mode, 
-                               centroid, meanfun, minfun, maxfun, meandom, mindom, maxdom, dfrange, modindx, startdom, enddom, dfslope)) else
-                                 return(data.frame(sound.files = X$sound.files[i], selec = X$selec[i], duration, meanfreq, sd, median, Q25, Q75, IQR, skew, kurt, sp.ent, sfm, mode, 
-                                                   centroid, peakf, meanfun, minfun, maxfun, meandom, mindom, maxdom, dfrange, modindx, startdom, enddom, dfslope))
+    if(fast) return(data.frame(sound.files = X$sound.files[i], selec = X$selec[i], duration, meanfreq, sd, freq.median, freq.Q25, freq.Q75, freq.IQR, time.median, time.Q25, time.Q75, time.IQR, skew, kurt, sp.ent, time.ent, entropy, sfm, 
+                               meanfun, minfun, maxfun, meandom, mindom, maxdom, dfrange, modindx, startdom, enddom, dfslope)) else
+                                 return(data.frame(sound.files = X$sound.files[i], selec = X$selec[i], duration, meanfreq, sd, freq.median, freq.Q25, freq.Q75, freq.IQR, time.median, time.Q25, time.Q75, time.IQR, skew, kurt, sp.ent, time.ent, entropy, sfm, 
+                                                 peakf, meanfun, minfun, maxfun, meandom, mindom, maxdom, dfrange, modindx, startdom, enddom, dfslope))
   }
   
+  if(any(parallel == 1, Sys.info()[1] == "Linux") & pb) message("measuring acoustic parameters:")
+  
   # Run parallel in windows
   if(parallel > 1) {if(Sys.info()[1] == "Windows") {
     
@@ -226,8 +279,12 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
     
     
   } else {    # Run parallel in other operating systems
-    
-    sp <- parallel::mclapply(1:nrow(X), function(i) {
+  
+    if(pb)  
+      sp <- pbmcapply::pbmclapply(1:nrow(X), mc.cores = parallel, function(i) {
+        spFUN(X = X, i = i, bp = bp, wl = wl, threshold = threshold)
+      }) else    
+    sp <- parallel::mclapply(1:nrow(X), mc.cores = parallel, function(i) {
       spFUN(X = X, i = i, bp = bp, wl = wl, threshold = threshold)
     })
     
@@ -244,5 +301,5 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
   row.names(sp) <- 1:nrow(sp)
   
   return(sp)
-  if(!is.null(path)) on.exit(setwd(wd))
+  if(!is.null(path)) setwd(wd)
   }
